@@ -1069,6 +1069,8 @@ gcm_prefs_devices_treeview_clicked_cb (GtkTreeSelection *selection,
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                                                "toolbutton_device_default"));
   gtk_widget_set_visible (widget, profile != NULL);
+  if (profile)
+    gtk_widget_set_sensitive (widget, !cd_profile_get_is_system_wide (profile));
   widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
                                                "toolbutton_device_add"));
   gtk_widget_set_visible (widget, FALSE);
@@ -2280,6 +2282,12 @@ gcm_prefs_window_realize_cb (GtkWidget *widget, CcColorPanel *prefs)
   prefs->priv->main_window = gtk_widget_get_toplevel (widget);
 }
 
+static const char *
+cc_color_panel_get_help_uri (CcPanel *panel)
+{
+  return "help:gnome-help/color";
+}
+
 static void
 cc_color_panel_get_property (GObject    *object,
                               guint       property_id,
@@ -2356,9 +2364,12 @@ static void
 cc_color_panel_class_init (CcColorPanelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  CcPanelClass *panel_class = CC_PANEL_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (CcColorPanelPrivate));
 
+  panel_class->get_help_uri = cc_color_panel_get_help_uri;
+
   object_class->get_property = cc_color_panel_get_property;
   object_class->set_property = cc_color_panel_set_property;
   object_class->dispose = cc_color_panel_dispose;
