@@ -1292,6 +1292,7 @@ on_extra_options_button_clicked (GtkWidget    *button,
   gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                 GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Other Media"));
   g_signal_connect (dialog,
                     "response",
                     G_CALLBACK (on_extra_options_dialog_response),
@@ -1348,7 +1349,9 @@ info_panel_setup_media (CcInfoPanel *self)
     { "x-content/video-hddvd", N_("HD DVD video disc") },
     { "x-content/image-picturecd", N_("Picture CD") },
     { "x-content/video-svcd", N_("Super Video CD") },
-    { "x-content/video-vcd", N_("Video CD") }
+    { "x-content/video-vcd", N_("Video CD") },
+    { "x-content/win32-software", N_("Windows software") },
+    { "x-content/software", N_("Software") }
   };
 
   for (n = 0; n < G_N_ELEMENTS (defs); n++) {
