@@ -67,9 +67,7 @@ enum
   SECTION_N_COLUMNS
 };
 
-static guint maybe_block_accels_id = 0;
-static gboolean block_accels = FALSE;
-static GSettings *settings = NULL;
+static GSettings *binding_settings = NULL;
 static GtkWidget *custom_shortcut_dialog = NULL;
 static GtkWidget *custom_shortcut_name_entry = NULL;
 static GtkWidget *custom_shortcut_command_entry = NULL;
@@ -528,7 +526,7 @@ append_sections_from_gsettings (GtkBuilder *builder)
   /* load custom shortcuts from GSettings */
   entries = g_array_new (FALSE, TRUE, sizeof (KeyListEntry));
 
-  custom_paths = g_settings_get_strv (settings, "custom-keybindings");
+  custom_paths = g_settings_get_strv (binding_settings, "custom-keybindings");
   for (i = 0; custom_paths[i]; i++)
     {
       key.name = g_strdup (custom_paths[i]);
@@ -795,13 +793,6 @@ section_selection_changed (GtkTreeSelection *selection, gpointer data)
     }
 }
 
-typedef struct
-{
-  GtkTreeView *tree_view;
-  GtkTreePath *path;
-  GtkTreeViewColumn *column;
-} IdleData;
-
 static gboolean
 edit_custom_shortcut (CcKeyboardItem *item)
 {
@@ -869,12 +860,12 @@ remove_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
   g_settings_apply (item->settings);
   g_settings_sync ();
 
-  settings_paths = g_settings_get_strv (settings, "custom-keybindings");
+  settings_paths = g_settings_get_strv (binding_settings, "custom-keybindings");
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
   for (i = 0; settings_paths[i]; i++)
     if (strcmp (settings_paths[i], item->gsettings_path) != 0)
       g_variant_builder_add (&builder, "s", settings_paths[i]);
-  g_settings_set_value (settings,
+  g_settings_set_value (binding_settings,
                         "custom-keybindings", g_variant_builder_end (&builder));
   g_strfreev (settings_paths);
   g_object_unref (item);
@@ -910,19 +901,6 @@ update_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
     }
 }
 
-static gboolean
-real_start_editing_cb (IdleData *idle_data)
-{
-  gtk_widget_grab_focus (GTK_WIDGET (idle_data->tree_view));
-  gtk_tree_view_set_cursor (idle_data->tree_view,
-                            idle_data->path,
-                            idle_data->column,
-                            TRUE);
-  gtk_tree_path_free (idle_data->path);
-  g_free (idle_data);
-  return FALSE;
-}
-
 static gboolean
 start_editing_cb (GtkTreeView    *tree_view,
                   GdkEventButton *event,
@@ -940,17 +918,10 @@ start_editing_cb (GtkTreeView    *tree_view,
                                      &path, &column,
                                      NULL, NULL))
     {
-      IdleData *idle_data;
       GtkTreeModel *model;
       GtkTreeIter iter;
       CcKeyboardItem *item;
 
-      if (gtk_tree_path_get_depth (path) == 1)
-        {
-          gtk_tree_path_free (path);
-          return FALSE;
-        }
-
       model = gtk_tree_view_get_model (tree_view);
       gtk_tree_model_get_iter (model, &iter, path);
       gtk_tree_model_get (model, &iter,
@@ -970,15 +941,15 @@ start_editing_cb (GtkTreeView    *tree_view,
         }
       else
         {
-          idle_data = g_new (IdleData, 1);
-          idle_data->tree_view = tree_view;
-          idle_data->path = path;
-          idle_data->column = item->desc_editable ? column :
-                              gtk_tree_view_get_column (tree_view, 1);
-          g_idle_add ((GSourceFunc) real_start_editing_cb, idle_data);
-          block_accels = TRUE;
+          gtk_widget_grab_focus (GTK_WIDGET (tree_view));
+          gtk_tree_view_set_cursor (tree_view,
+                                    path,
+                                    item->desc_editable ? column :
+                                    gtk_tree_view_get_column (tree_view, 1),
+                                    TRUE);
         }
       g_signal_stop_emission_by_name (tree_view, "button_press_event");
+      gtk_tree_path_free (path);
     }
   return TRUE;
 }
@@ -999,16 +970,6 @@ start_editing_kb_cb (GtkTreeView *treeview,
                       DETAIL_KEYENTRY_COLUMN, &item,
                       -1);
 
-  if (item == NULL)
-    {
-      /* This is a section heading - expand or collapse */
-      if (gtk_tree_view_row_expanded (treeview, path))
-        gtk_tree_view_collapse_row (treeview, path);
-      else
-        gtk_tree_view_expand_row (treeview, path, FALSE);
-      return;
-    }
-
   /* if only the accel can be edited on the selected row
    * always select the accel column */
   if (item->desc_editable &&
@@ -1142,8 +1103,6 @@ accel_edited_callback (GtkCellRendererText   *cell,
   CcKeyboardItem *item;
   char *str;
 
-  block_accels = FALSE;
-
   model = gtk_tree_view_get_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_path_free (path);
@@ -1178,7 +1137,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
     }
 
   /* Check for unmodified keys */
-  if (mask == 0 && keycode != 0)
+  if ((mask == 0 || mask == GDK_SHIFT_MASK) && keycode != 0)
     {
       if ((keyval >= GDK_KEY_a && keyval <= GDK_KEY_z)
            || (keyval >= GDK_KEY_A && keyval <= GDK_KEY_Z)
@@ -1293,8 +1252,6 @@ accel_cleared_callback (GtkCellRendererText *cell,
   GtkTreeIter iter;
   GtkTreeModel *model;
 
-  block_accels = FALSE;
-
   model = gtk_tree_view_get_model (view);
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_path_free (path);
@@ -1310,21 +1267,6 @@ accel_cleared_callback (GtkCellRendererText *cell,
   g_object_set (G_OBJECT (item), "binding", "", NULL);
 }
 
-/* this handler is used to keep accels from activating while the user
- * is assigning a new shortcut so that he won't accidentally trigger one
- * of the widgets */
-static gboolean
-maybe_block_accels (GtkWidget *widget,
-                    GdkEventKey *event,
-                    gpointer user_data)
-{
-  if (block_accels)
-  {
-    return gtk_window_propagate_key_event (GTK_WINDOW (widget), event);
-  }
-  return FALSE;
-}
-
 static gchar *
 find_free_settings_path ()
 {
@@ -1332,7 +1274,7 @@ find_free_settings_path ()
   char *dir = NULL;
   int i, num, n_names;
 
-  used_names = g_settings_get_strv (settings, "custom-keybindings");
+  used_names = g_settings_get_strv (binding_settings, "custom-keybindings");
   n_names = g_strv_length (used_names);
 
   for (num = 0; dir == NULL; num++)
@@ -1392,12 +1334,12 @@ add_custom_shortcut (GtkTreeView  *tree_view,
       gtk_list_store_append (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter, DETAIL_KEYENTRY_COLUMN, item, -1);
 
-      settings_paths = g_settings_get_strv (settings, "custom-keybindings");
+      settings_paths = g_settings_get_strv (binding_settings, "custom-keybindings");
       g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
       for (i = 0; settings_paths[i]; i++)
         g_variant_builder_add (&builder, "s", settings_paths[i]);
       g_variant_builder_add (&builder, "s", item->gsettings_path);
-      g_settings_set_value (settings, "custom-keybindings",
+      g_settings_set_value (binding_settings, "custom-keybindings",
                             g_variant_builder_end (&builder));
 
       /* make the new shortcut visible */
@@ -1663,7 +1605,7 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
   treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder,
                                                     "shortcut_treeview"));
 
-  settings = g_settings_new (BINDINGS_SCHEMA);
+  binding_settings = g_settings_new (BINDINGS_SCHEMA);
 
   g_signal_connect (treeview, "button_press_event",
                     G_CALLBACK (start_editing_cb), builder);
@@ -1675,7 +1617,6 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
 
   column = gtk_tree_view_column_new_with_attributes (_("Action"),
                                                      renderer,
-                                                     "text", DETAIL_DESCRIPTION_COLUMN,
                                                      NULL);
   gtk_tree_view_column_set_cell_data_func (column, renderer, description_set_func, NULL, NULL);
   gtk_tree_view_column_set_resizable (column, FALSE);
@@ -1723,9 +1664,6 @@ setup_dialog (CcPanel *panel, GtkBuilder *builder)
   shell = cc_panel_get_shell (CC_PANEL (panel));
   widget = cc_shell_get_toplevel (shell);
 
-  maybe_block_accels_id = g_signal_connect (widget, "key_press_event",
-                                            G_CALLBACK (maybe_block_accels), NULL);
-
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
   g_signal_connect (selection, "changed",
                     G_CALLBACK (shortcut_selection_changed),
@@ -1770,31 +1708,21 @@ keyboard_shortcuts_init (CcPanel *panel, GtkBuilder *builder)
 void
 keyboard_shortcuts_dispose (CcPanel *panel)
 {
-  if (maybe_block_accels_id != 0)
+  if (kb_system_sections != NULL)
     {
-      CcShell *shell;
-      GtkWidget *toplevel;
-
-      shell = cc_panel_get_shell (CC_PANEL (panel));
-      toplevel = cc_shell_get_toplevel (shell);
-
-      g_signal_handler_disconnect (toplevel, maybe_block_accels_id);
-      maybe_block_accels_id = 0;
-
-      if (kb_system_sections != NULL)
-        {
-          g_hash_table_destroy (kb_system_sections);
-          kb_system_sections = NULL;
-        }
-      if (kb_apps_sections != NULL)
-        {
-          g_hash_table_destroy (kb_apps_sections);
-          kb_apps_sections = NULL;
-        }
-      if (kb_user_sections != NULL)
-        {
-          g_hash_table_destroy (kb_user_sections);
-          kb_user_sections = NULL;
-        }
+      g_hash_table_destroy (kb_system_sections);
+      kb_system_sections = NULL;
     }
+  if (kb_apps_sections != NULL)
+    {
+      g_hash_table_destroy (kb_apps_sections);
+      kb_apps_sections = NULL;
+    }
+  if (kb_user_sections != NULL)
+    {
+      g_hash_table_destroy (kb_user_sections);
+      kb_user_sections = NULL;
+    }
+
+  g_clear_object (&binding_settings);
 }
