@@ -25,37 +25,181 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gio/gdesktopappinfo.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-xkb-info.h>
 
+#ifdef HAVE_IBUS
+#include <ibus.h>
+#endif
+
+#include "gdm-languages.h"
 #include "gnome-region-panel-input.h"
 
 #define WID(s) GTK_WIDGET(gtk_builder_get_object (builder, s))
 
 #define GNOME_DESKTOP_INPUT_SOURCES_DIR "org.gnome.desktop.input-sources"
 
-#define KEY_INPUT_SOURCES "sources"
+#define KEY_CURRENT_INPUT_SOURCE "current"
+#define KEY_INPUT_SOURCES        "sources"
 
-#define INPUT_SOURCE_TYPE_XKB "xkb"
+#define INPUT_SOURCE_TYPE_XKB  "xkb"
+#define INPUT_SOURCE_TYPE_IBUS "ibus"
 
 enum {
   NAME_COLUMN,
   TYPE_COLUMN,
   ID_COLUMN,
+  SETUP_COLUMN,
   N_COLUMNS
 };
 
 static GSettings *input_sources_settings = NULL;
-static gulong input_sources_settings_changed_id = 0;
 static GnomeXkbInfo *xkb_info = NULL;
 
+#ifdef HAVE_IBUS
+static IBusBus *ibus = NULL;
+static GHashTable *ibus_engines = NULL;
+
+static const gchar *supported_ibus_engines[] = {
+  "bopomofo",
+  "pinyin",
+  "chewing",
+  "anthy",
+  "hangul",
+  NULL
+};
+#endif  /* HAVE_IBUS */
+
 static GtkWidget *input_chooser_new          (GtkWindow     *main_window,
                                               GtkListStore  *active_sources);
 static gboolean   input_chooser_get_selected (GtkWidget     *chooser,
                                               GtkTreeModel **model,
                                               GtkTreeIter   *iter);
 
+#ifdef HAVE_IBUS
+static void
+clear_ibus (void)
+{
+  g_clear_pointer (&ibus_engines, g_hash_table_destroy);
+  g_clear_object (&ibus);
+}
+
+static gchar *
+engine_get_display_name (IBusEngineDesc *engine_desc)
+{
+  const gchar *name;
+  const gchar *language_code;
+  gchar *language;
+  gchar *display_name;
+
+  name = ibus_engine_desc_get_longname (engine_desc);
+  language_code = ibus_engine_desc_get_language (engine_desc);
+
+  language = gdm_get_language_from_name (language_code, NULL);
+
+  display_name = g_strdup_printf ("%s (%s)", language, name);
+
+  g_free (language);
+
+  return display_name;
+}
+
+static GDesktopAppInfo *
+setup_app_info_for_id (const gchar *id)
+{
+  GDesktopAppInfo *app_info;
+  gchar *desktop_file_name;
+
+  desktop_file_name = g_strdup_printf ("ibus-setup-%s.desktop", id);
+  app_info = g_desktop_app_info_new (desktop_file_name);
+  g_free (desktop_file_name);
+
+  return app_info;
+}
+
+static void
+update_ibus_active_sources (GtkBuilder *builder)
+{
+  GtkTreeView *tv;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gchar *type, *id;
+
+  tv = GTK_TREE_VIEW (WID ("active_input_sources"));
+  model = gtk_tree_view_get_model (tv);
+
+  gtk_tree_model_get_iter_first (model, &iter);
+  do
+    {
+      gtk_tree_model_get (model, &iter,
+                          TYPE_COLUMN, &type,
+                          ID_COLUMN, &id,
+                          -1);
+
+      if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS))
+        {
+          IBusEngineDesc *engine_desc = NULL;
+          gchar *display_name = NULL;
+
+          engine_desc = g_hash_table_lookup (ibus_engines, id);
+          if (engine_desc)
+            {
+              display_name = engine_get_display_name (engine_desc);
+
+              gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                                  NAME_COLUMN, display_name,
+                                  -1);
+              g_free (display_name);
+            }
+        }
+
+      g_free (type);
+      g_free (id);
+    }
+  while (gtk_tree_model_iter_next (model, &iter));
+}
+
+static void
+fetch_ibus_engines (GtkBuilder *builder)
+{
+  IBusEngineDesc **engines, **iter;
+  const gchar *name;
+
+  ibus_engines = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, g_object_unref);
+
+  engines = ibus_bus_get_engines_by_names (ibus, supported_ibus_engines);
+  for (iter = engines; *iter; ++iter)
+    {
+      name = ibus_engine_desc_get_name (*iter);
+      g_hash_table_replace (ibus_engines, (gpointer)name, *iter);
+    }
+
+  g_free (engines);
+
+  update_ibus_active_sources (builder);
+
+  /* We've got everything we needed, don't want to be called again. */
+  g_signal_handlers_disconnect_by_func (ibus, fetch_ibus_engines, builder);
+}
+
+static void
+maybe_start_ibus (void)
+{
+  /* IBus doesn't export API in the session bus. The only thing
+   * we have there is a well known name which we can use as a
+   * sure-fire way to activate it. */
+  g_bus_unwatch_name (g_bus_watch_name (G_BUS_TYPE_SESSION,
+                                        IBUS_SERVICE_IBUS,
+                                        G_BUS_NAME_WATCHER_FLAGS_AUTO_START,
+                                        NULL,
+                                        NULL,
+                                        NULL,
+                                        NULL));
+}
+#endif  /* HAVE_IBUS */
+
 static gboolean
 add_source_to_table (GtkTreeModel *model,
                      GtkTreePath  *path,
@@ -118,6 +262,39 @@ populate_model (GtkListStore *store,
   g_free (source_id);
 
   g_list_free (sources);
+
+#ifdef HAVE_IBUS
+  if (ibus_engines)
+    {
+      gchar *display_name;
+
+      sources = g_hash_table_get_keys (ibus_engines);
+
+      source_id = NULL;
+      for (tmp = sources; tmp; tmp = tmp->next)
+        {
+          g_free (source_id);
+          source_id = g_strconcat (INPUT_SOURCE_TYPE_IBUS, tmp->data, NULL);
+
+          if (g_hash_table_contains (active_sources_table, source_id))
+            continue;
+
+          display_name = engine_get_display_name (g_hash_table_lookup (ibus_engines, tmp->data));
+
+          gtk_list_store_append (store, &iter);
+          gtk_list_store_set (store, &iter,
+                              NAME_COLUMN, display_name,
+                              TYPE_COLUMN, INPUT_SOURCE_TYPE_IBUS,
+                              ID_COLUMN, tmp->data,
+                              -1);
+          g_free (display_name);
+        }
+      g_free (source_id);
+
+      g_list_free (sources);
+    }
+#endif
+
   g_hash_table_destroy (active_sources_table);
 }
 
@@ -129,6 +306,8 @@ populate_with_active_sources (GtkListStore *store)
   const gchar *name;
   const gchar *type;
   const gchar *id;
+  gchar *display_name;
+  GDesktopAppInfo *app_info;
   GtkTreeIter tree_iter;
 
   sources = g_settings_get_value (input_sources_settings, KEY_INPUT_SOURCES);
@@ -136,26 +315,52 @@ populate_with_active_sources (GtkListStore *store)
   g_variant_iter_init (&iter, sources);
   while (g_variant_iter_next (&iter, "(&s&s)", &type, &id))
     {
-      if (!g_str_equal (type, INPUT_SOURCE_TYPE_XKB))
+      display_name = NULL;
+      app_info = NULL;
+
+      if (g_str_equal (type, INPUT_SOURCE_TYPE_XKB))
+        {
+          gnome_xkb_info_get_layout_info (xkb_info, id, &name, NULL, NULL, NULL);
+          if (!name)
+            {
+              g_warning ("Couldn't find XKB input source '%s'", id);
+              continue;
+            }
+          display_name = g_strdup (name);
+        }
+      else if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS))
+        {
+#ifdef HAVE_IBUS
+          IBusEngineDesc *engine_desc = NULL;
+
+          if (ibus_engines)
+            engine_desc = g_hash_table_lookup (ibus_engines, id);
+
+          if (engine_desc)
+            display_name = engine_get_display_name (engine_desc);
+
+          app_info = setup_app_info_for_id (id);
+#else
+          g_warning ("IBus input source type specified but IBus support was not compiled");
+          continue;
+#endif
+        }
+      else
         {
           g_warning ("Unknown input source type '%s'", type);
           continue;
         }
 
-      gnome_xkb_info_get_layout_info (xkb_info, id, &name, NULL, NULL, NULL);
-
-      if (!name)
-        {
-          g_warning ("Couldn't find XKB input source '%s'", id);
-          continue;
-        }
-
       gtk_list_store_append (store, &tree_iter);
       gtk_list_store_set (store, &tree_iter,
-                          NAME_COLUMN, name,
+                          NAME_COLUMN, display_name ? display_name : id,
                           TYPE_COLUMN, type,
                           ID_COLUMN, id,
+                          SETUP_COLUMN, app_info,
                           -1);
+      g_free (display_name);
+      if (app_info)
+        g_object_unref (app_info);
     }
 
   g_variant_unref (sources);
@@ -168,23 +373,59 @@ update_configuration (GtkTreeModel *model)
   gchar *type;
   gchar *id;
   GVariantBuilder builder;
+  GVariant *old_sources;
+  const gchar *old_current_type;
+  const gchar *old_current_id;
+  guint old_current_index;
+  guint index;
+
+  old_sources = g_settings_get_value (input_sources_settings, KEY_INPUT_SOURCES);
+  old_current_index = g_settings_get_uint (input_sources_settings, KEY_CURRENT_INPUT_SOURCE);
+  g_variant_get_child (old_sources,
+                       old_current_index,
+                       "(&s&s)",
+                       &old_current_type,
+                       &old_current_id);
+  if (g_variant_n_children (old_sources) < 1)
+    {
+      g_warning ("No input source configured, resetting");
+      g_settings_reset (input_sources_settings, KEY_INPUT_SOURCES);
+      goto exit;
+    }
+  if (old_current_index >= g_variant_n_children (old_sources))
+    {
+      g_settings_set_uint (input_sources_settings,
+                           KEY_CURRENT_INPUT_SOURCE,
+                           g_variant_n_children (old_sources) - 1);
+    }
 
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("a(ss)"));
-
+  index = 0;
   gtk_tree_model_get_iter_first (model, &iter);
-  do {
-    gtk_tree_model_get (model, &iter,
-                        TYPE_COLUMN, &type,
-                        ID_COLUMN, &id,
-                        -1);
-    g_variant_builder_add (&builder, "(ss)", type, id);
-    g_free (type);
-    g_free (id);
-  } while (gtk_tree_model_iter_next (model, &iter));
+  do
+    {
+      gtk_tree_model_get (model, &iter,
+                          TYPE_COLUMN, &type,
+                          ID_COLUMN, &id,
+                          -1);
+      if (index != old_current_index &&
+          g_str_equal (type, old_current_type) &&
+          g_str_equal (id, old_current_id))
+        {
+          g_settings_set_uint (input_sources_settings, KEY_CURRENT_INPUT_SOURCE, index);
+        }
+      g_variant_builder_add (&builder, "(ss)", type, id);
+      g_free (type);
+      g_free (id);
+      index += 1;
+    }
+  while (gtk_tree_model_iter_next (model, &iter));
 
-  g_signal_handler_block (input_sources_settings, input_sources_settings_changed_id);
   g_settings_set_value (input_sources_settings, KEY_INPUT_SOURCES, g_variant_builder_end (&builder));
-  g_signal_handler_unblock (input_sources_settings, input_sources_settings_changed_id);
+
+ exit:
+  g_settings_apply (input_sources_settings);
+  g_variant_unref (old_sources);
 }
 
 static gboolean
@@ -200,17 +441,13 @@ get_selected_iter (GtkBuilder    *builder,
 }
 
 static gint
-find_selected_layout_idx (GtkBuilder *builder)
+idx_from_model_iter (GtkTreeModel *model,
+                     GtkTreeIter  *iter)
 {
-  GtkTreeIter iter;
-  GtkTreeModel *model;
   GtkTreePath *path;
   gint idx;
 
-  if (!get_selected_iter (builder, &model, &iter))
-    return -1;
-
-  path = gtk_tree_model_get_path (model, &iter);
+  path = gtk_tree_model_get_path (model, iter);
   if (path == NULL)
     return -1;
 
@@ -227,24 +464,45 @@ update_button_sensitivity (GtkBuilder *builder)
   GtkWidget *up_button;
   GtkWidget *down_button;
   GtkWidget *show_button;
+  GtkWidget *settings_button;
   GtkTreeView *tv;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
   gint n_active;
   gint index;
+  gboolean settings_sensitive;
+  GDesktopAppInfo *app_info;
 
   remove_button = WID("input_source_remove");
   show_button = WID("input_source_show");
   up_button = WID("input_source_move_up");
   down_button = WID("input_source_move_down");
+  settings_button = WID("input_source_settings");
 
   tv = GTK_TREE_VIEW (WID ("active_input_sources"));
-
   n_active = gtk_tree_model_iter_n_children (gtk_tree_view_get_model (tv), NULL);
-  index = find_selected_layout_idx (builder);
+
+  if (get_selected_iter (builder, &model, &iter))
+    {
+      index = idx_from_model_iter (model, &iter);
+      gtk_tree_model_get (model, &iter, SETUP_COLUMN, &app_info, -1);
+    }
+  else
+    {
+      index = -1;
+      app_info = NULL;
+    }
+
+  settings_sensitive = (index >= 0 && app_info != NULL);
+
+  if (app_info)
+    g_object_unref (app_info);
 
   gtk_widget_set_sensitive (remove_button, index >= 0 && n_active > 1);
   gtk_widget_set_sensitive (show_button, index >= 0);
   gtk_widget_set_sensitive (up_button, index > 0);
   gtk_widget_set_sensitive (down_button, index >= 0 && index < n_active - 1);
+  gtk_widget_set_sensitive (settings_button, settings_sensitive);
 }
 
 static void
@@ -270,12 +528,13 @@ chooser_response (GtkWidget *chooser, gint response_id, gpointer data)
 
       if (input_chooser_get_selected (chooser, &model, &iter))
         {
-          GtkTreeView *my_tv;
+          GtkTreeView *tv;
           GtkListStore *my_model;
-          GtkTreeIter my_iter;
+          GtkTreeIter child_iter;
           gchar *name;
           gchar *type;
           gchar *id;
+          GDesktopAppInfo *app_info = NULL;
 
           gtk_tree_model_get (model, &iter,
                               NAME_COLUMN, &name,
@@ -283,21 +542,27 @@ chooser_response (GtkWidget *chooser, gint response_id, gpointer data)
                               ID_COLUMN, &id,
                               -1);
 
-          my_tv = GTK_TREE_VIEW (WID ("active_input_sources"));
-          my_model = GTK_LIST_STORE (gtk_tree_view_get_model (my_tv));
+#ifdef HAVE_IBUS
+          if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS))
+            app_info = setup_app_info_for_id (id);
+#endif
 
-          gtk_list_store_append (my_model, &my_iter);
+          tv = GTK_TREE_VIEW (WID ("active_input_sources"));
+          my_model = GTK_LIST_STORE (gtk_tree_view_get_model (tv));
 
-          gtk_list_store_set (my_model, &my_iter,
-                              NAME_COLUMN, name,
-                              TYPE_COLUMN, type,
-                              ID_COLUMN, id,
-                              -1);
+          gtk_list_store_insert_with_values (my_model, &child_iter, -1,
+                                             NAME_COLUMN, name,
+                                             TYPE_COLUMN, type,
+                                             ID_COLUMN, id,
+                                             SETUP_COLUMN, app_info,
+                                             -1);
           g_free (name);
           g_free (type);
           g_free (id);
+          if (app_info)
+            g_object_unref (app_info);
 
-          gtk_tree_selection_select_iter (gtk_tree_view_get_selection (my_tv), &my_iter);
+          gtk_tree_selection_select_iter (gtk_tree_view_get_selection (tv), &child_iter);
 
           update_button_sensitivity (builder);
           update_configuration (GTK_TREE_MODEL (my_model));
@@ -345,6 +610,7 @@ remove_selected_input (GtkButton *button, gpointer data)
     return;
 
   path = gtk_tree_model_get_path (model, &iter);
+
   gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 
   if (!gtk_tree_model_get_iter (model, &iter, path))
@@ -378,6 +644,7 @@ move_selected_input_up (GtkButton *button, gpointer data)
   path = gtk_tree_model_get_path (model, &prev);
 
   gtk_list_store_swap (GTK_LIST_STORE (model), &iter, &prev);
+
   set_selected_path (builder, path);
   gtk_tree_path_free (path);
 
@@ -403,7 +670,9 @@ move_selected_input_down (GtkButton *button, gpointer data)
     return;
 
   path = gtk_tree_model_get_path (model, &next);
+
   gtk_list_store_swap (GTK_LIST_STORE (model), &iter, &next);
+
   set_selected_path (builder, path);
   gtk_tree_path_free (path);
 
@@ -417,6 +686,7 @@ show_selected_layout (GtkButton *button, gpointer data)
   GtkBuilder *builder = data;
   GtkTreeModel *model;
   GtkTreeIter iter;
+  gchar *type;
   gchar *id;
   gchar *kbd_viewer_args;
   const gchar *xkb_layout;
@@ -428,14 +698,46 @@ show_selected_layout (GtkButton *button, gpointer data)
     return;
 
   gtk_tree_model_get (model, &iter,
+                      TYPE_COLUMN, &type,
                       ID_COLUMN, &id,
                       -1);
 
-  gnome_xkb_info_get_layout_info (xkb_info, id, NULL, NULL, &xkb_layout, &xkb_variant);
-
-  if (!xkb_layout || !xkb_layout[0])
+  if (g_str_equal (type, INPUT_SOURCE_TYPE_XKB))
     {
-      g_warning ("Couldn't find XKB input source '%s'", id);
+      gnome_xkb_info_get_layout_info (xkb_info, id, NULL, NULL, &xkb_layout, &xkb_variant);
+
+      if (!xkb_layout || !xkb_layout[0])
+        {
+          g_warning ("Couldn't find XKB input source '%s'", id);
+          goto exit;
+        }
+    }
+  else if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS))
+    {
+#ifdef HAVE_IBUS
+      IBusEngineDesc *engine_desc = NULL;
+
+      if (ibus_engines)
+        engine_desc = g_hash_table_lookup (ibus_engines, id);
+
+      if (engine_desc)
+        {
+          xkb_layout = ibus_engine_desc_get_layout (engine_desc);
+          xkb_variant = "";
+        }
+      else
+        {
+          g_warning ("Couldn't find IBus input source '%s'", id);
+          goto exit;
+        }
+#else
+      g_warning ("IBus input source type specified but IBus support was not compiled");
+      goto exit;
+#endif
+    }
+  else
+    {
+      g_warning ("Unknown input source type '%s'", type);
       goto exit;
     }
 
@@ -450,9 +752,43 @@ show_selected_layout (GtkButton *button, gpointer data)
 
   g_free (kbd_viewer_args);
  exit:
+  g_free (type);
   g_free (id);
 }
 
+static void
+show_selected_settings (GtkButton *button, gpointer data)
+{
+  GtkBuilder *builder = data;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  GdkAppLaunchContext *ctx;
+  GDesktopAppInfo *app_info;
+  GError *error = NULL;
+
+  g_debug ("show selected layout");
+
+  if (!get_selected_iter (builder, &model, &iter))
+    return;
+
+  gtk_tree_model_get (model, &iter, SETUP_COLUMN, &app_info, -1);
+
+  if (!app_info)
+    return;
+
+  ctx = gdk_display_get_app_launch_context (gdk_display_get_default ());
+  gdk_app_launch_context_set_timestamp (ctx, gtk_get_current_event_time ());
+
+  if (!g_app_info_launch (G_APP_INFO (app_info), NULL, G_APP_LAUNCH_CONTEXT (ctx), &error))
+    {
+      g_warning ("Failed to launch input source setup: %s", error->message);
+      g_error_free (error);
+    }
+
+  g_object_unref (ctx);
+  g_object_unref (app_info);
+}
+
 static gboolean
 go_to_shortcuts (GtkLinkButton *button,
                  CcRegionPanel *panel)
@@ -478,11 +814,26 @@ input_sources_changed (GSettings  *settings,
 {
   GtkWidget *treeview;
   GtkTreeModel *store;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  GtkTreeModel *model;
 
   treeview = WID("active_input_sources");
   store = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+
+  if (get_selected_iter (builder, &model, &iter))
+    path = gtk_tree_model_get_path (model, &iter);
+  else
+    path = NULL;
+
   gtk_list_store_clear (GTK_LIST_STORE (store));
   populate_with_active_sources (GTK_LIST_STORE (store));
+
+  if (path)
+    {
+      set_selected_path (builder, path);
+      gtk_tree_path_free (path);
+    }
 }
 
 static void
@@ -541,7 +892,23 @@ setup_input_tabs (GtkBuilder    *builder,
   GtkTreeSelection *selection;
 
   input_sources_settings = g_settings_new (GNOME_DESKTOP_INPUT_SOURCES_DIR);
-  xkb_info = gnome_xkb_info_new ();
+  g_settings_delay (input_sources_settings);
+  g_object_weak_ref (G_OBJECT (builder), (GWeakNotify) g_object_unref, input_sources_settings);
+
+  if (!xkb_info)
+    xkb_info = gnome_xkb_info_new ();
+
+#ifdef HAVE_IBUS
+  ibus_init ();
+  if (!ibus)
+    {
+      ibus = ibus_bus_new_async ();
+      g_signal_connect_swapped (ibus, "connected",
+                                G_CALLBACK (fetch_ibus_engines), builder);
+      g_object_weak_ref (G_OBJECT (builder), (GWeakNotify) clear_ibus, NULL);
+    }
+  maybe_start_ibus ();
+#endif
 
   /* set up the list of active inputs */
   treeview = WID("active_input_sources");
@@ -551,7 +918,11 @@ setup_input_tabs (GtkBuilder    *builder,
   gtk_tree_view_column_add_attribute (column, cell, "text", NAME_COLUMN);
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
-  store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+  store = gtk_list_store_new (N_COLUMNS,
+                              G_TYPE_STRING,
+                              G_TYPE_STRING,
+                              G_TYPE_STRING,
+                              G_TYPE_DESKTOP_APP_INFO);
 
   populate_with_active_sources (store);
 
@@ -572,6 +943,8 @@ setup_input_tabs (GtkBuilder    *builder,
                     G_CALLBACK (move_selected_input_down), builder);
   g_signal_connect (WID("input_source_show"), "clicked",
                     G_CALLBACK (show_selected_layout), builder);
+  g_signal_connect (WID("input_source_settings"), "clicked",
+                    G_CALLBACK (show_selected_settings), builder);
 
   /* use an em dash is no shortcut */
   update_shortcuts (builder);
@@ -579,10 +952,10 @@ setup_input_tabs (GtkBuilder    *builder,
   g_signal_connect (WID("jump-to-shortcuts"), "activate-link",
                     G_CALLBACK (go_to_shortcuts), panel);
 
-  input_sources_settings_changed_id = g_signal_connect (G_OBJECT (input_sources_settings),
-                                                        "changed::" KEY_INPUT_SOURCES,
-                                                        G_CALLBACK (input_sources_changed),
-                                                        builder);
+  g_signal_connect (G_OBJECT (input_sources_settings),
+                    "changed::" KEY_INPUT_SOURCES,
+                    G_CALLBACK (input_sources_changed),
+                    builder);
 }
 
 static void
