@@ -539,6 +539,18 @@ cc_screen_panel_init (CcScreenPanel *self)
                    widget, "sensitive",
                    G_SETTINGS_BIND_GET);
 
+  widget = WID ("show_notifications_check");
+  g_settings_bind (self->priv->lock_settings,
+                   "show-notifications",
+                   widget, "active",
+                   G_SETTINGS_BIND_DEFAULT);
+
+  widget = WID ("show_notifications_box");
+  g_settings_bind (self->priv->lock_settings,
+                   "lock-enabled",
+                   widget, "sensitive",
+                   G_SETTINGS_BIND_GET);
+
   update_lock_screen_sensitivity (self);
 
   widget = WID ("screen_vbox");
